import * as i0 from '@angular/core';
import { Directive, Optional, Self, Input, HostListener, Injectable, NgModule } from '@angular/core';
import { fromEvent, merge } from 'rxjs';
import _Inputmask from 'inputmask';
import * as i1 from '@angular/cdk/platform';
import * as i2 from '@angular/forms';
import { formatDate, getCurrencySymbol, getLocaleNumberSymbol, NumberSymbol, getNumberOfCurrencyDigits } from '@angular/common';
import * as i1$1 from '@bravobit/bb-foundation/localize';

const createMask = (options) => typeof options === 'string' ? { mask: options } : options;

const InputmaskConstructor = _Inputmask.default || _Inputmask;
class BbInputMask {
    set mask(mask) {
        if (!mask || typeof mask === 'string') {
            return;
        }
        this._inputMaskOptions = mask;
        this.updateMask();
    }
    constructor(_ngZone, _platform, _renderer, _elementRef, _control) {
        this._ngZone = _ngZone;
        this._platform = _platform;
        this._renderer = _renderer;
        this._elementRef = _elementRef;
        this._control = _control;
        // State.
        this.inputMaskPlugin = null;
        this.nativeInputElement = null;
        this.eventsSubscription = null;
        this._inputMaskOptions = null;
        this._onChange = () => ({});
        this.onInput = (_) => ({});
        this.onTouched = () => ({});
        this.validate = (control) => !control.value || !this.inputMaskPlugin || this.inputMaskPlugin?.isValid()
            ? null
            : { inputMask: true };
        this.nativeInputElement = this._elementRef.nativeElement;
        if (this._control != null) {
            this._control.valueAccessor = this;
        }
    }
    ngOnInit() {
        const control = this._control?.control;
        if (!control) {
            return;
        }
        control?.setValidators(control.validator ? [control.validator, this.validate] : [this.validate]);
        control?.updateValueAndValidity();
    }
    ngOnDestroy() {
        this.destroyMask();
    }
    writeValue(value) {
        if (!this.nativeInputElement) {
            return;
        }
        this._renderer.setProperty(this.nativeInputElement, 'value', value ?? '');
    }
    registerOnChange(fn) {
        this._onChange = fn;
        const parser = this._inputMaskOptions?.parser;
        this.onInput = value => this._onChange(parser ? parser(value) : value);
    }
    registerOnTouched(fn) {
        this.onTouched = fn;
    }
    setDisabledState(isDisabled) {
        if (!this.nativeInputElement) {
            return;
        }
        this._renderer.setProperty(this.nativeInputElement, 'disabled', isDisabled);
    }
    updateMask() {
        this.destroyMask();
        this.createMask();
        this.registerOnChange(this._onChange);
    }
    createMask() {
        if (!this._platform.isBrowser || !this.nativeInputElement || this._inputMaskOptions === null || Object.keys(this._inputMaskOptions).length === 0) {
            return;
        }
        const { parser, ...options } = this._inputMaskOptions;
        this.inputMaskPlugin = this._ngZone.runOutsideAngular(() => new InputmaskConstructor(options).mask(this.nativeInputElement));
        const setValue$ = fromEvent(this.nativeInputElement, 'setvalue');
        const cleared$ = fromEvent(this.nativeInputElement, 'cleared');
        this.eventsSubscription = merge(setValue$, cleared$)
            .subscribe(event => this.onInput(event?.target?.['value']));
        const control = this._control?.control;
        if (!control) {
            return;
        }
        setTimeout(() => control?.updateValueAndValidity());
    }
    destroyMask() {
        this.eventsSubscription?.unsubscribe();
        this.eventsSubscription = null;
        this.inputMaskPlugin?.remove();
        this.inputMaskPlugin = null;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.0.5", ngImport: i0, type: BbInputMask, deps: [{ token: i0.NgZone }, { token: i1.Platform }, { token: i0.Renderer2 }, { token: i0.ElementRef }, { token: i2.NgControl, optional: true, self: true }], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "18.0.5", type: BbInputMask, isStandalone: true, selector: "input[bbInputMask],input[bbCurrencyMask],input[bbDateMask]", inputs: { mask: ["bbInputMask", "mask"] }, host: { listeners: { "input": "onInput($event.target.value)", "blur": "onTouched($event.target.value)" } }, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.0.5", ngImport: i0, type: BbInputMask, decorators: [{
            type: Directive,
            args: [{
                    selector: 'input[bbInputMask],input[bbCurrencyMask],input[bbDateMask]',
                    standalone: true
                }]
        }], ctorParameters: () => [{ type: i0.NgZone }, { type: i1.Platform }, { type: i0.Renderer2 }, { type: i0.ElementRef }, { type: i2.NgControl, decorators: [{
                    type: Optional
                }, {
                    type: Self
                }] }], propDecorators: { mask: [{
                type: Input,
                args: ['bbInputMask']
            }], onInput: [{
                type: HostListener,
                args: ['input', ['$event.target.value']]
            }], onTouched: [{
                type: HostListener,
                args: ['blur', ['$event.target.value']]
            }] } });

class Masking {
    constructor(_localize) {
        this._localize = _localize;
    }
    date(format, separator = '-') {
        const dateParts = this.getDateParts(format);
        const inputFormat = dateParts.join(separator);
        const outputFormat = 'yyyy-MM-dd';
        const escape = /[.*+?^${}()\/|[\]\\]/g;
        const escapedSeparator = separator.replace(escape, '\\$&');
        const inputFormatRegexString = dateParts
            .map(item => `(?<${item}>[0-9]{${item?.length}})`)
            .join(escapedSeparator);
        const inputFormatRegex = new RegExp(`^${inputFormatRegexString}$`);
        return createMask({
            alias: 'datetime',
            inputFormat: inputFormat.toLowerCase(),
            outputFormat: outputFormat.toLowerCase(),
            nullable: false,
            inputmode: 'numeric',
            onBeforeMask: (initialValue) => {
                return initialValue ? formatDate(initialValue, inputFormat, 'en-US') : null;
            },
            onBeforePaste: (value) => {
                return value;
            },
            parser: (value) => {
                if (!value || typeof value !== 'string' || !inputFormatRegex.test(value)) {
                    return null;
                }
                const matches = value.match(inputFormatRegex);
                const year = +matches?.groups?.['yyyy'];
                const month = +matches?.groups?.['MM'] - 1;
                const day = +matches?.groups?.['dd'];
                const date = new Date(year, month, day);
                return date instanceof Date && !isNaN(date)
                    ? formatDate(date, outputFormat, 'en-US')
                    : null;
            }
        });
    }
    currency(code, options) {
        const locale = this._localize?.current?.locale ?? 'en';
        const symbol = options?.symbol ?? getCurrencySymbol(code, options?.symbolFormat ?? 'narrow', locale);
        const decimal = options?.decimalCharacter ?? getLocaleNumberSymbol(locale, NumberSymbol.CurrencyDecimal);
        const group = options?.groupCharacter ?? getLocaleNumberSymbol(locale, NumberSymbol.CurrencyGroup);
        const digits = options?.digits ?? getNumberOfCurrencyDigits(code);
        const spacer = options?.spacer ?? ' ';
        return createMask({
            alias: 'numeric',
            digits: digits,
            allowMinus: options?.allowMinus ?? false,
            min: options?.min ?? 0,
            max: options?.max ?? Number.MAX_SAFE_INTEGER,
            prefix: `${symbol}${spacer}`,
            radixPoint: decimal ?? '.',
            groupSeparator: group ?? ',',
            digitsOptional: options?.digitsOptional ?? false,
            nullable: options?.nullable ?? true,
            autoUnmask: true,
            unmaskAsNumber: true,
            rightAlign: false,
            showMaskOnHover: false,
            showMaskOnFocus: false,
            inputmode: digits > 0 ? 'decimal' : 'numeric',
            parser: (value) => {
                if (value === '') {
                    return null;
                }
                return value;
            }
        });
    }
    getDateParts(format) {
        switch (format) {
            case 'day-month-year':
                return ['dd', 'MM', 'yyyy'];
            case 'month-day-year':
                return ['MM', 'dd', 'yyyy'];
            case 'year-day-month':
                return ['yyyy', 'dd', 'MM'];
            case 'year-month-day':
            default:
                return ['yyyy', 'MM', 'dd'];
        }
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.0.5", ngImport: i0, type: Masking, deps: [{ token: i1$1.Localize, optional: true }], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "18.0.5", ngImport: i0, type: Masking, providedIn: 'root' }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.0.5", ngImport: i0, type: Masking, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: () => [{ type: i1$1.Localize, decorators: [{
                    type: Optional
                }] }] });

class BbCurrencyMask {
    constructor(_masking, _inputMask) {
        this._masking = _masking;
        this._inputMask = _inputMask;
        // Inputs.
        this.currencyCode = null;
        this.options = null;
    }
    ngOnChanges(changes) {
        const currencyCode = changes?.['currencyCode']?.currentValue ?? this.currencyCode ?? 'USD';
        const options = changes?.['options']?.currentValue ?? this.options ?? {};
        this._inputMask.mask = this._masking.currency(currencyCode, options);
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.0.5", ngImport: i0, type: BbCurrencyMask, deps: [{ token: Masking }, { token: BbInputMask, self: true }], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "18.0.5", type: BbCurrencyMask, isStandalone: true, selector: "input[bbCurrencyMask]", inputs: { currencyCode: ["bbCurrencyMask", "currencyCode"], options: ["bbCurrencyMaskOptions", "options"] }, usesOnChanges: true, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.0.5", ngImport: i0, type: BbCurrencyMask, decorators: [{
            type: Directive,
            args: [{
                    selector: 'input[bbCurrencyMask]',
                    standalone: true
                }]
        }], ctorParameters: () => [{ type: Masking }, { type: BbInputMask, decorators: [{
                    type: Self
                }] }], propDecorators: { currencyCode: [{
                type: Input,
                args: ['bbCurrencyMask']
            }], options: [{
                type: Input,
                args: ['bbCurrencyMaskOptions']
            }] } });

class BbDateMask {
    constructor(_masking, _inputMask) {
        this._masking = _masking;
        this._inputMask = _inputMask;
        // Inputs.
        this.format = null;
        this.separator = null;
    }
    ngOnChanges(changes) {
        const format = changes?.['format']?.currentValue ?? this.format ?? null;
        const separator = changes?.['separator']?.currentValue ?? this.separator ?? '-';
        this._inputMask.mask = this._masking.date(format, separator);
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.0.5", ngImport: i0, type: BbDateMask, deps: [{ token: Masking }, { token: BbInputMask, self: true }], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "18.0.5", type: BbDateMask, isStandalone: true, selector: "input[bbDateMask]", inputs: { format: ["bbDateMask", "format"], separator: ["bbDateMaskSeparator", "separator"] }, usesOnChanges: true, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.0.5", ngImport: i0, type: BbDateMask, decorators: [{
            type: Directive,
            args: [{
                    selector: 'input[bbDateMask]',
                    standalone: true
                }]
        }], ctorParameters: () => [{ type: Masking }, { type: BbInputMask, decorators: [{
                    type: Self
                }] }], propDecorators: { format: [{
                type: Input,
                args: ['bbDateMask']
            }], separator: [{
                type: Input,
                args: ['bbDateMaskSeparator']
            }] } });

class MaskingModule {
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.0.5", ngImport: i0, type: MaskingModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule }); }
    static { this.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "18.0.5", ngImport: i0, type: MaskingModule, imports: [BbInputMask,
            BbCurrencyMask,
            BbDateMask], exports: [BbInputMask,
            BbCurrencyMask,
            BbDateMask] }); }
    static { this.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "18.0.5", ngImport: i0, type: MaskingModule }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.0.5", ngImport: i0, type: MaskingModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [
                        BbInputMask,
                        BbCurrencyMask,
                        BbDateMask
                    ],
                    exports: [
                        BbInputMask,
                        BbCurrencyMask,
                        BbDateMask
                    ]
                }]
        }] });

/**
 * Generated bundle index. Do not edit.
 */

export { BbCurrencyMask, BbDateMask, BbInputMask, Masking, MaskingModule, createMask };
//# sourceMappingURL=bravobit-bb-foundation-masking.mjs.map
