import * as i0 from '@angular/core';
import { Injectable, InjectionToken, inject, Optional, Inject } from '@angular/core';
import * as i1 from '@angular/cdk/platform';
import { DOCUMENT, formatDate } from '@angular/common';
import { map, distinctUntilChanged, shareReplay, debounceTime, startWith } from 'rxjs/operators';
import { of, fromEvent, Observable, merge, Subscription, isObservable } from 'rxjs';

class FileLoader {
    constructor() {
        this.isFile = (input) => {
            return 'File' in window && input instanceof File;
        };
        this.isBlob = (input) => {
            return 'Blob' in window && input instanceof Blob;
        };
        this.blobToFile = (blob, fileName) => {
            const anyBlob = blob;
            anyBlob.lastModifiedDate = new Date();
            anyBlob.name = fileName;
            return anyBlob;
        };
    }
    fromDataUrl(value, name = 'file') {
        if (!value) {
            return null;
        }
        try {
            // Convert base64 to raw binary data held in a string.
            const byteString = window.atob(value.split(',')[1]);
            // Separate out the mime component.
            const mimeString = value
                .split(',')[0]
                .split(':')[1]
                .split(';')[0];
            // Write the bytes of the string to an ArrayBuffer.
            const arrayBuffer = new ArrayBuffer(byteString.length);
            const uint8Array = new Uint8Array(arrayBuffer);
            for (let index = 0; index < byteString.length; index++) {
                uint8Array[index] = byteString.charCodeAt(index);
            }
            const dataView = new DataView(arrayBuffer);
            const blob = new Blob([dataView.buffer], { type: mimeString });
            return this.createFile(blob, name);
        }
        catch {
            return null;
        }
    }
    asDataUrl(file) {
        return this.read(file, 'dataurl');
    }
    asArrayBuffer(file) {
        return this.read(file, 'arraybuffer');
    }
    async read(file, type) {
        return new Promise((resolve, reject) => {
            const isFileLike = this.isFile(file) || this.isBlob(file);
            if (!file || !isFileLike) {
                return reject('Invalid file');
            }
            const reader = new FileReader();
            reader.onerror = () => reject('Error file read');
            reader.onabort = () => reject('Abort file read');
            reader.onload = () => {
                const result = reader.result;
                return resolve(result);
            };
            if (type === 'dataurl') {
                reader.readAsDataURL(file);
            }
            else if (type === 'arraybuffer') {
                reader.readAsArrayBuffer(file);
            }
        });
    }
    createFile(blob, name) {
        if (!navigator['msSaveBlob']) {
            return new File([blob], name, { lastModified: Date.now(), type: blob?.type });
        }
        const blobFile = new Blob([blob], { type: blob?.type });
        return this.blobToFile(blobFile, name);
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.0.5", ngImport: i0, type: FileLoader, deps: [], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "18.0.5", ngImport: i0, type: FileLoader, providedIn: 'root' }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.0.5", ngImport: i0, type: FileLoader, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }] });

class Exif {
    constructor(_platform, _fileLoader) {
        this._platform = _platform;
        this._fileLoader = _fileLoader;
        this.transformImage = (image, type, orientation, quality = 100) => {
            // Create the canvas and context.
            const canvas = document.createElement('canvas');
            const context = canvas.getContext('2d');
            // Validate the context exists.
            if (!context) {
                return null;
            }
            // Set the canvas size based on the image.
            const { width, height } = image;
            canvas.width = width;
            canvas.height = height;
            // Validate the orientation is correct
            // else flip the canvas sizes.
            if (4 < orientation && orientation < 9) {
                canvas.width = height;
                canvas.height = width;
            }
            // Transform the context based on the orientation.
            switch (orientation) {
                case 2:
                    context.transform(-1, 0, 0, 1, width, 0);
                    break;
                case 3:
                    context.transform(-1, 0, 0, -1, width, height);
                    break;
                case 4:
                    context.transform(1, 0, 0, -1, 0, height);
                    break;
                case 5:
                    context.transform(0, 1, 1, 0, 0, 0);
                    break;
                case 6:
                    context.transform(0, 1, -1, 0, height, 0);
                    break;
                case 7:
                    context.transform(0, -1, -1, 0, height, width);
                    break;
                case 8:
                    context.transform(0, -1, 1, 0, 0, width);
                    break;
                default:
                    break;
            }
            // Draw the image on the context and return the data URL.
            context.drawImage(image, 0, 0, width, height);
            return canvas.toDataURL(type, quality / 100);
        };
        this.getSupportedCanvasType = (fileType) => {
            switch (fileType) {
                case 'image/jpeg':
                case 'image/jpg':
                    return 'image/jpeg';
                case 'image/x-windows-bmp':
                case 'image/bmp':
                    return 'image/bmp';
                default:
                    return 'image/png';
            }
        };
        this.getOrientation = (buffer) => {
            // Create a new data view.
            const view = new DataView(buffer);
            if (view.getUint16(0, false) !== 0xFFD8) {
                return -2;
            }
            const length = view.byteLength;
            let offset = 2;
            while (offset < length) {
                if (view.getUint16(offset + 2, false) <= 8) {
                    return -1;
                }
                const marker = view.getUint16(offset, false);
                offset += 2;
                if (marker === 0xFFE1) {
                    if (view.getUint32(offset += 2, false) !== 0x45786966) {
                        return -1;
                    }
                    const little = view.getUint16(offset += 6, false) === 0x4949;
                    offset += view.getUint32(offset + 4, little);
                    const tags = view.getUint16(offset, little);
                    offset += 2;
                    for (let index = 0; index < tags; index++) {
                        if (view.getUint16(offset + index * 12, little) === 0x0112) {
                            return view.getUint16(offset + index * 12 + 8, little);
                        }
                    }
                }
                else if ((marker & 0xFF00) !== 0xFF00) {
                    break;
                }
                else {
                    offset += view.getUint16(offset, false);
                }
            }
            return -1;
        };
    }
    async strip(file, quality = 100) {
        // Validate we are on a browser.
        if (!this._platform.isBrowser) {
            return file;
        }
        // Validate the file exists and is
        // an instance of a File object.
        if (!file || !(file instanceof File)) {
            return file;
        }
        // Only jpeg images need correction.
        if (!['image/jpeg'].includes(file.type)) {
            return file;
        }
        try {
            // Read the file as an array buffer and data url.
            const arrayBuffer = await this._fileLoader.asArrayBuffer(file);
            const dataUrl = await this._fileLoader.asDataUrl(file);
            // Create an image element.
            const imageElement = await this.createImageElement(dataUrl);
            // Get the orientation by using the buffer.
            const orientation = this.getOrientation(arrayBuffer);
            // Transform the image to remove the orientation.
            const canvasType = this.getSupportedCanvasType(file.type);
            const imageUrl = this.transformImage(imageElement, canvasType, orientation, quality);
            // Validate the image url exists.
            if (!imageUrl) {
                return file;
            }
            // Transform the image url to a file.
            return this._fileLoader.fromDataUrl(imageUrl, file.name);
        }
        catch {
            // When something goes wrong we
            // just return the default file.
            return file;
        }
    }
    async createImageElement(dataUrl) {
        return new Promise((resolve, reject) => {
            const image = new Image();
            image.src = dataUrl;
            image.onerror = () => reject('Error image create');
            image.onabort = () => reject('Abort image create');
            image.onload = () => resolve(image);
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.0.5", ngImport: i0, type: Exif, deps: [{ token: i1.Platform }, { token: FileLoader }], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "18.0.5", ngImport: i0, type: Exif, providedIn: 'root' }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.0.5", ngImport: i0, type: Exif, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: () => [{ type: i1.Platform }, { type: FileLoader }] });

class ImageConverter {
    constructor(_exif, _platform, _fileLoader) {
        this._exif = _exif;
        this._platform = _platform;
        this._fileLoader = _fileLoader;
        // Data.
        this._types = ['image/png', 'image/jpeg', 'image/webp'];
        this.calculateOffsetAndDimensions = (image, canvasWidth, canvasHeight) => {
            const originalWidth = image.width;
            const originalHeight = image.height;
            const widthRatio = canvasWidth / originalWidth;
            const heightRatio = canvasHeight / originalHeight;
            let width;
            let height;
            if (originalWidth > originalHeight) {
                height = originalHeight * heightRatio;
                width = (originalWidth / originalHeight) * height;
            }
            else {
                width = originalWidth * widthRatio;
                height = (originalHeight / originalWidth) * width;
            }
            let xOffset = 0;
            let yOffset = 0;
            if (canvasHeight === height) {
                xOffset = -((width - canvasWidth) / 2);
            }
            else if (canvasWidth === width) {
                yOffset = -((height - canvasHeight) / 2);
            }
            return { xOffset, yOffset, width, height };
        };
    }
    async toDataUri(contents, options = null) {
        // Check if the current platform is a browser.
        if (!this._platform.isBrowser) {
            return null;
        }
        // Get the image.
        const image = await this.getImage(contents);
        // Get the canvas width and height.
        const canvasWidth = options?.width || image?.width;
        const canvasHeight = options?.height || image?.height;
        // Get the canvas and context.
        const canvas = document.createElement('canvas');
        const context = canvas.getContext('2d');
        // Validate the context exists.
        if (!context) {
            return null;
        }
        // Set the width and height.
        canvas.width = canvasWidth;
        canvas.height = canvasHeight;
        // When the canvas sizes are 0 we have failed.
        if (canvas?.width === 0 && canvas?.height === 0) {
            return null;
        }
        // Check if dimensions where supplied, else create the image.
        if (!options) {
            // Draw the image.
            context.drawImage(image, 0, 0);
            // Convert the canvas to a data url.
            return canvas.toDataURL(this._types[0]);
        }
        // Get the dimensions and offsets.
        const { width, height, xOffset, yOffset } = this.calculateOffsetAndDimensions(image, canvasWidth, canvasHeight);
        // Draw the image.
        context.drawImage(image, 0, 0, image.width, image.height, xOffset, yOffset, width, height);
        // Convert the canvas to a data url.
        return canvas.toDataURL(this._types[0]);
    }
    async fileToImage(file, types = this._types) {
        // Check if the file matches an image.
        if (!types.includes(file.type)) {
            throw new Error('The selected file does not meet the required format.');
        }
        let stripped = null;
        try {
            stripped = await this._exif.strip(file, 90);
        }
        catch {
            // Do nothing.
        }
        // Ge the data uri.
        const dataUri = await this._fileLoader.asDataUrl(stripped ?? file);
        // Convert the data URI to a image element.
        return await this.stringToImage(dataUri);
    }
    stringToImage(src) {
        return new Promise((resolve, reject) => {
            const image = new Image();
            image.src = src;
            image.onload = () => resolve(image);
            image.onerror = error => reject(error);
        });
    }
    async getImage(contents) {
        // If the contents is already an HTML image element just return it.
        if (contents instanceof HTMLImageElement) {
            return contents;
        }
        // If the contents is a file convert it to an HTML image element.
        if (contents instanceof File) {
            return this.fileToImage(contents);
        }
        // If the contents is a string convert it to an HTML image element.
        return this.stringToImage(contents);
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.0.5", ngImport: i0, type: ImageConverter, deps: [{ token: Exif }, { token: i1.Platform }, { token: FileLoader }], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "18.0.5", ngImport: i0, type: ImageConverter, providedIn: 'root' }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.0.5", ngImport: i0, type: ImageConverter, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: () => [{ type: Exif }, { type: i1.Platform }, { type: FileLoader }] });

const WINDOW = new InjectionToken('An abstraction over global window object', {
    factory: () => {
        const { defaultView } = inject(DOCUMENT);
        if (!defaultView) {
            throw new Error('Window is not available');
        }
        return defaultView;
    }
});

class Files {
    constructor(_platform, _rendererFactory, _window) {
        this._platform = _platform;
        this._rendererFactory = _rendererFactory;
        this._window = _window;
        this._renderer = this._rendererFactory.createRenderer(null, null);
    }
    download(blobOrFile, name) {
        if (!this._platform.isBrowser) {
            return;
        }
        const anchor = this.createAnchorElement(blobOrFile, name);
        this.revokeUrlAndPressAnchor(anchor);
    }
    fromDataUri(dataUri, name) {
        if (!dataUri) {
            return null;
        }
        try {
            const byteString = this._window.atob(dataUri.split(',')[1]);
            // Separate out the mime component.
            const mimeString = dataUri
                .split(',')[0]
                .split(':')[1]
                .split(';')[0];
            // Write the bytes of the string to an ArrayBuffer.
            const arrayBuffer = new ArrayBuffer(byteString.length);
            const uint8Array = new Uint8Array(arrayBuffer);
            for (let index = 0; index < byteString.length; index++) {
                uint8Array[index] = byteString.charCodeAt(index);
            }
            const dataView = new DataView(arrayBuffer);
            const blob = new Blob([dataView.buffer], { type: mimeString });
            return this.create(blob, name);
        }
        catch {
            return null;
        }
    }
    create(blob, name) {
        if (!navigator['msSaveBlob']) {
            return new File([blob], name, { lastModified: Date.now(), type: blob?.type });
        }
        const anyBlob = new Blob([blob], { type: blob?.type });
        anyBlob.lastModifiedDate = new Date();
        anyBlob.name = name;
        return anyBlob;
    }
    createAnchorElement(blobOrFile, name) {
        const anchor = this._renderer.createElement('a');
        this._renderer.setProperty(anchor, 'download', name ?? blobOrFile?.['name']);
        this._renderer.setProperty(anchor, 'rel', 'noopener');
        this._renderer.setProperty(anchor, 'href', URL.createObjectURL(blobOrFile));
        return anchor;
    }
    revokeUrlAndPressAnchor(anchor) {
        setTimeout(() => URL.revokeObjectURL(anchor?.href), 40_000);
        setTimeout(() => this.pressAnchor(anchor), 0);
    }
    pressAnchor(anchor) {
        try {
            anchor.dispatchEvent(new MouseEvent('click'));
        }
        catch {
            anchor.click();
        }
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.0.5", ngImport: i0, type: Files, deps: [{ token: i1.Platform }, { token: i0.RendererFactory2 }, { token: WINDOW, optional: true }], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "18.0.5", ngImport: i0, type: Files, providedIn: 'root' }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.0.5", ngImport: i0, type: Files, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: () => [{ type: i1.Platform }, { type: i0.RendererFactory2 }, { type: Window, decorators: [{
                    type: Optional
                }, {
                    type: Inject,
                    args: [WINDOW]
                }] }] });

const ACCEPT_LANGUAGE = new InjectionToken('AcceptLanguage');

class Languages {
    constructor(_acceptLanguage) {
        this._acceptLanguage = _acceptLanguage;
        // Data.
        this._data = [];
        this.getLanguages = () => {
            // If defined we must be in a server environment.
            if (this._acceptLanguage) {
                return this._acceptLanguage;
            }
            // Try to get the languages from the navigator.
            if (typeof navigator !== 'object') {
                return null;
            }
            let t = 'anguage';
            let n = navigator;
            let f = n['l' + t + 's'];
            const data = f && f.length ? f : (t = n['l' + t] ||
                n['browserL' + t] ||
                n['userL' + t]) ? [t] : t;
            return typeof data === 'string'
                ? data
                : data.join(';');
        };
        this._data = this.initialize();
    }
    get all() {
        return this._data;
    }
    initialize() {
        // Get the languages.
        const data = this.getLanguages();
        // Parse all languages by using a regex.
        const strings = (data || '')
            .match(/((([a-zA-Z]+(-[a-zA-Z0-9]+){0,2})|\*)(;q=[0-1](\.[0-9]+)?)?)*/g);
        // Get the content by parsing the strings.
        const content = strings.map(item => {
            if (!item) {
                return null;
            }
            const bits = item.split(';');
            const ietf = bits[0].split('-');
            const hasScript = ietf.length === 3;
            return {
                code: ietf[0],
                script: hasScript ? ietf[1] : null,
                region: hasScript ? ietf[2] : ietf[1],
                quality: bits[1] ? parseFloat(bits[1].split('=')[1]) : 1.0
            };
        });
        // Filter out all the empty items
        // and sort them by the quality.
        return content
            .filter(item => !!item)
            .sort((a, b) => b.quality - a.quality);
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.0.5", ngImport: i0, type: Languages, deps: [{ token: ACCEPT_LANGUAGE, optional: true }], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "18.0.5", ngImport: i0, type: Languages, providedIn: 'root' }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.0.5", ngImport: i0, type: Languages, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: () => [{ type: undefined, decorators: [{
                    type: Optional
                }, {
                    type: Inject,
                    args: [ACCEPT_LANGUAGE]
                }] }] });

const NAVIGATOR = new InjectionToken('An abstraction over window.navigator object', {
    factory: () => inject(WINDOW).navigator
});

class Network {
    constructor(_platform, _window, _navigator) {
        this._platform = _platform;
        this._window = _window;
        this._navigator = _navigator;
        // Data.
        this._online$ = of(true);
        this.getOnlineObservable();
    }
    online() {
        return this._online$;
    }
    getOnlineObservable() {
        // Validate we are on a browser.
        if (!this._platform.isBrowser) {
            return;
        }
        // Get all the events from the window.
        const online$ = fromEvent(this._window, 'online').pipe(map(() => true));
        const offline$ = fromEvent(this._window, 'offline').pipe(map(() => false));
        const now$ = new Observable(subscriber => {
            subscriber.next(this._navigator?.onLine);
            subscriber.complete();
        });
        // Merge all event so we get notified when
        // a user is online/offline.
        this._online$ = merge(now$, online$, offline$).pipe(distinctUntilChanged(), shareReplay({ refCount: true, bufferSize: 1 }));
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.0.5", ngImport: i0, type: Network, deps: [{ token: i1.Platform }, { token: WINDOW, optional: true }, { token: NAVIGATOR, optional: true }], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "18.0.5", ngImport: i0, type: Network, providedIn: 'root' }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.0.5", ngImport: i0, type: Network, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: () => [{ type: i1.Platform }, { type: Window, decorators: [{
                    type: Optional
                }, {
                    type: Inject,
                    args: [WINDOW]
                }] }, { type: Navigator, decorators: [{
                    type: Optional
                }, {
                    type: Inject,
                    args: [NAVIGATOR]
                }] }] });

class Patch {
    constructor(_platform, _window, _document) {
        this._platform = _platform;
        this._window = _window;
        this._document = _document;
        this._hasPatchedMobileVerticalHeight = false;
        // Subscriptions.
        this._subscription = new Subscription();
    }
    mobileVerticalHeight() {
        // Check if the user has already patched
        // the mobile vertical height.
        if (this._hasPatchedMobileVerticalHeight) {
            return;
        }
        this.periodicallySetDocumentVerticalProperty();
        this._hasPatchedMobileVerticalHeight = true;
    }
    ngOnDestroy() {
        this._subscription?.unsubscribe();
    }
    periodicallySetDocumentVerticalProperty() {
        // Validate we are using a browser.
        if (!this._platform.isBrowser) {
            return;
        }
        // Listen to a debounced window resize event.
        const resize$ = fromEvent(this._window, 'resize').pipe(debounceTime(25), startWith(0));
        // Subscribe to the resize observable.
        const subscription = resize$.subscribe(() => {
            const verticalHeight = this._window?.innerHeight * 0.01;
            this._document?.documentElement?.style?.setProperty('--vh', `${verticalHeight}px`);
        });
        // Save the subscription so we can destroy it later.
        this._subscription.add(subscription);
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.0.5", ngImport: i0, type: Patch, deps: [{ token: i1.Platform }, { token: WINDOW, optional: true }, { token: DOCUMENT, optional: true }], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "18.0.5", ngImport: i0, type: Patch, providedIn: 'root' }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.0.5", ngImport: i0, type: Patch, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: () => [{ type: i1.Platform }, { type: Window, decorators: [{
                    type: Optional
                }, {
                    type: Inject,
                    args: [WINDOW]
                }] }, { type: Document, decorators: [{
                    type: Optional
                }, {
                    type: Inject,
                    args: [DOCUMENT]
                }] }] });

const BASE_URL = new InjectionToken('BaseUrl');

const COOKIE = new InjectionToken('Cookie');

const LOCATION = new InjectionToken('An abstraction over window.location object', {
    factory: () => inject(WINDOW).location
});

const BbRegex = {
    email: /^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/,
    fullNumber: /^\d*[1-9]\d*$/,
    url: new RegExp('^' +
        // protocol identifier (optional)
        // short syntax // still required
        '(?:(?:(?:https?|ftp):)?\\/\\/)' +
        // user:pass BasicAuth (optional)
        '(?:\\S+(?::\\S*)?@)?' +
        '(?:' +
        // IP address exclusion
        // private & local networks
        '(?!(?:10|127)(?:\\.\\d{1,3}){3})' +
        '(?!(?:169\\.254|192\\.168)(?:\\.\\d{1,3}){2})' +
        '(?!172\\.(?:1[6-9]|2\\d|3[0-1])(?:\\.\\d{1,3}){2})' +
        // IP address dotted notation octets
        // excludes loopback network 0.0.0.0
        // excludes reserved space >= 224.0.0.0
        // excludes network & broadcast addresses
        // (first & last IP address of each class)
        '(?:[1-9]\\d?|1\\d\\d|2[01]\\d|22[0-3])' +
        '(?:\\.(?:1?\\d{1,2}|2[0-4]\\d|25[0-5])){2}' +
        '(?:\\.(?:[1-9]\\d?|1\\d\\d|2[0-4]\\d|25[0-4]))' +
        '|' +
        // host & domain names, may end with dot
        // can be replaced by a shortest alternative
        // (?![-_])(?:[-\\w\\u00a1-\\uffff]{0,63}[^-_]\\.)+
        '(?:' +
        '(?:' +
        '[a-z0-9\\u00a1-\\uffff]' +
        '[a-z0-9\\u00a1-\\uffff_-]{0,62}' +
        ')?' +
        '[a-z0-9\\u00a1-\\uffff]\\.' +
        ')+' +
        // TLD identifier name, may end with dot
        '(?:[a-z\\u00a1-\\uffff]{2,}\\.?)' +
        ')' +
        // port number (optional)
        '(?::\\d{2,5})?' +
        // resource path (optional)
        '(?:[/?#]\\S*)?' +
        '$', 'i')
};

class BbValidator {
    static email(control) {
        if (isEmptyInputValue(control.value)) {
            return null;
        }
        return BbRegex.email.test(control.value) ? null : { email: true };
    }
    static fullNumber(control) {
        if (isEmptyInputValue(control.value)) {
            return null;
        }
        return BbRegex.fullNumber.test(control.value) ? null : { fullNumber: true };
    }
    static maxFileSize(bytes) {
        const validator = (control) => {
            const value = control.value;
            // Validate if the value is a file.
            if (!(value instanceof File)) {
                return null;
            }
            if (value.size <= bytes) {
                return null;
            }
            let maxSize = '0 Bytes';
            if (bytes !== 0) {
                const k = 1024;
                const dm = 2;
                const sizes = ['Bytes', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB'];
                const index = Math.floor(Math.log(bytes) / Math.log(k));
                maxSize = parseFloat((bytes / Math.pow(k, index)).toFixed(dm)) + ' ' + sizes[index];
            }
            return { maxFileSize: { maxSize } };
        };
        return validator;
    }
    static confirm(fieldName, error = 'confirm') {
        const validator = (control) => {
            if (isEmptyInputValue(control.value)) {
                return null;
            }
            const field = control?.parent?.get(fieldName);
            return control?.value === field?.value
                ? null
                : { [error]: true };
        };
        return validator;
    }
    static maxDate(date, error = 'maxDate') {
        const formattedDate = date.toString() !== 'Invalid Date'
            ? formatDate(date, 'yyyy-MM-dd', 'en-US')
            : null;
        const validator = (control) => {
            if (isEmptyInputValue(control.value)) {
                return null;
            }
            const controlDate = new Date(control.value);
            return controlDate.getTime() > date.getTime()
                ? { [error]: { date: formattedDate } }
                : null;
        };
        return validator;
    }
    static minDate(date, error = 'minDate') {
        const formattedDate = date.toString() !== 'Invalid Date'
            ? formatDate(date, 'yyyy-MM-dd', 'en-US')
            : null;
        const validator = (control) => {
            if (isEmptyInputValue(control.value)) {
                return null;
            }
            const controlDate = new Date(control.value);
            return controlDate.getTime() < date.getTime()
                ? { [error]: { date: formattedDate } }
                : null;
        };
        return validator;
    }
}
function isEmptyInputValue(value) {
    // We don't check for string here so it also works with arrays.
    return value == null || value.length === 0;
}

function observableAttribute(value) {
    return isObservable(value) ? value : of(value);
}

function parseDate(value) {
    if (value === undefined || value === null) {
        return null;
    }
    if (typeof value === 'string') {
        if (/^[0-9]{4}-[0-9]{2}-[0-9]{2}T[0-9]{2}:[0-9]{2}:[0-9]{2}$/.test(value)) {
            return createDate(new Date(`${value}Z`));
        }
        if (/^[0-9]{4}-[0-9]{2}-[0-9]{2}$/.test(value)) {
            const [y, m, d] = value.split('-');
            const year = Number(y);
            const month = Number(m) - 1;
            const day = Number(d);
            return createDate(new Date(year, month, day));
        }
    }
    return createDate(new Date(value));
}
function createDate(date) {
    return isValidDate(date) ? date : null;
}
function isValidDate(date) {
    if (!isDate(date)) {
        return false;
    }
    return !isNaN(Number(date));
}
function isDate(value) {
    return (value instanceof Date || (typeof value === 'object' && Object.prototype.toString.call(value) === '[object Date]'));
}

function generatePassword(length = 12) {
    const chars = 'abcdefghijklmnopqrstuvwxyz';
    const upperChars = chars.toUpperCase();
    const numberChars = '0123456789';
    const specialChars = '#?!@$%^&*-';
    const usableChars = chars + upperChars + numberChars + specialChars;
    let generatedPassword = '';
    for (let index = 0; index < length - 4; index++) {
        generatedPassword += usableChars[Math.floor(Math.random() * (usableChars.length))];
    }
    generatedPassword += chars[Math.floor(Math.random() * (chars.length))];
    generatedPassword += upperChars[Math.floor(Math.random() * (upperChars.length))];
    generatedPassword += numberChars[Math.floor(Math.random() * (numberChars.length))];
    generatedPassword += specialChars[Math.floor(Math.random() * (specialChars.length))];
    return generatedPassword;
}

function arrayToMap(items, value) {
    return items.reduce((previous, current) => {
        const key = current?.[value];
        return { ...previous, [key]: current };
    }, {});
}

/*
 * Public API Surface of bb-foundation
 */

/**
 * Generated bundle index. Do not edit.
 */

export { ACCEPT_LANGUAGE, BASE_URL, BbRegex, BbValidator, COOKIE, Exif, FileLoader, Files, ImageConverter, LOCATION, Languages, NAVIGATOR, Network, Patch, WINDOW, arrayToMap, createDate, generatePassword, isDate, isValidDate, observableAttribute, parseDate };
//# sourceMappingURL=bravobit-bb-foundation.mjs.map
